#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.guernsey import Guernsey
from tests.common import CommonCountryTests


class TestGG(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Guernsey)

    def test_special_holidays(self):
        dt = (
            "1930-10-10",
            "1935-05-06",
            "1937-05-12",
            "1939-09-04",
            "1949-09-19",
            "1953-06-02",
            "1957-07-26",
            "1977-06-07",
            "1978-06-28",
            "1981-07-29",
            "1989-05-23",
            "1999-12-28",
            "1999-12-31",
            "2000-01-03",
            "2001-07-12",
            "2002-06-03",
            "2011-04-29",
            "2012-06-05",
            "2022-06-03",
            "2022-09-19",
            "2023-05-08",
            "2024-07-16",
        )
        dt_observed = (
            "1932-12-27",
            "1933-01-02",
            "1938-12-27",
            "1939-01-02",
            "1981-12-28",
            "1982-12-28",
            "1983-01-03",
            "1987-12-28",
            "1993-12-28",
            "1994-01-03",
            "2004-12-28",
            "2005-01-03",
        )
        self.assertHoliday(dt, dt_observed)
        self.assertNoNonObservedHoliday(dt_observed)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_may_day_bank_holiday(self):
        name = "May Day Bank Holiday"
        self.assertHolidayName(
            name,
            "1995-05-08",
            "2020-05-08",
            "2021-05-03",
            "2022-05-02",
            "2023-05-01",
            "2024-05-06",
            "2025-05-05",
        )
        self.assertHolidayName(name, range(1980, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1980))

    def test_spring_bank_holiday(self):
        name = "Spring Bank Holiday"
        self.assertHolidayName(
            name,
            "1967-05-29",
            "1970-05-25",
            "1973-05-28",
            "1974-05-27",
            "1975-05-26",
            "1976-05-31",
            "2020-05-25",
            "2021-05-31",
            "2022-06-02",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(name, range(1979, self.end_year))
        self.assertNoHolidayName(
            name,
            range(self.start_year, 1967),
            range(1968, 1970),
            range(1971, 1973),
            range(1977, 1979),
        )

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "1966-05-30",
            "1968-06-03",
            "1969-05-26",
            "1971-05-31",
            "1972-05-22",
        )
        self.assertHolidayName(name, range(self.start_year, 1967))
        self.assertNoHolidayName(name, 1967, 1970, range(1973, self.end_year))

    def test_his_majestys_birthday(self):
        name = "His Majesty's Birthday"
        self.assertHolidayName(
            name,
            # Edward VII (MAY/JUN observance).
            "1909-06-25",
            # George V.
            "1910-06-24",
            "1911-05-27",
            "1912-06-14",
            "1913-06-03",
            "1914-06-24",
            # Not observed in 1915-1918 due to WW1.
            "1919-06-03",
            "1920-06-05",
            "1921-06-04",
            "1922-06-03",
            "1923-06-02",
            "1924-06-03",
            "1925-06-03",
            "1926-06-05",
            "1927-06-03",
            "1928-06-04",
            "1929-06-03",
            "1930-06-03",
            "1931-06-03",
            "1932-06-03",
            "1933-06-03",
            "1934-06-04",
            "1935-06-03",
            # Edward VIII.
            "1936-06-23",
            # George VI.
            "1937-06-09",
            "1938-06-09",
            "1939-06-08",
            # Not observed in 1940-1941 due to WW2.
            "1942-06-11",
            "1943-06-02",
            "1944-06-08",
            "1945-06-14",
            "1946-06-13",
        )
        self.assertNoHolidayName(
            name, range(1915, 1919), range(1940, 1942), range(1947, self.end_year)
        )

    def test_summer_bank_holiday(self):
        name = "Summer Bank Holiday"
        self.assertHolidayName(
            name,
            "1962-08-06",
            "1963-08-05",
            "1964-08-03",
            "1965-08-30",
            "1966-08-29",
            "1967-08-28",
            "1968-09-02",
            "1969-09-01",
            "2020-08-31",
            "2021-08-30",
            "2022-08-29",
            "2023-08-28",
            "2024-08-26",
            "2025-08-25",
        )
        self.assertHolidayName(name, range(self.start_year, 1977), range(1979, self.end_year))
        self.assertNoHolidayName(name, range(1977, 1979))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_liberation_day(self):
        name = "Liberation Day"
        self.assertHolidayName(
            name,
            "2010-05-10",
            (f"{year}-05-09" for year in (*range(1946, 2010), *range(2011, 2050))),
        )
        self.assertNoHolidayName(name, range(self.start_year, 1946))

    def test_2019(self):
        # Wayback Machine of https://www.gov.gg/holidaydates
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-06", "May Day Bank Holiday"),
            ("2019-05-09", "Liberation Day"),
            ("2019-05-27", "Spring Bank Holiday"),
            ("2019-08-26", "Summer Bank Holiday"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2020(self):
        # Wayback Machine of https://www.gov.gg/holidaydates
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-08", "May Day Bank Holiday"),
            ("2020-05-09", "Liberation Day"),
            ("2020-05-25", "Spring Bank Holiday"),
            ("2020-08-31", "Summer Bank Holiday"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
            ("2020-12-28", "Boxing Day (substitute day)"),
        )

    def test_2021(self):
        # Wayback Machine of https://www.gov.gg/holidaydates
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-03", "May Day Bank Holiday"),
            ("2021-05-09", "Liberation Day"),
            ("2021-05-31", "Spring Bank Holiday"),
            ("2021-08-30", "Summer Bank Holiday"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Christmas Day (substitute day)"),
            ("2021-12-28", "Boxing Day (substitute day)"),
        )

    def test_2022(self):
        # Wayback Machine of https://www.gov.gg/holidaydates
        # https://www.bbc.com/news/world-europe-guernsey-62864318
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (substitute day)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "May Day Bank Holiday"),
            ("2022-05-09", "Liberation Day"),
            ("2022-06-02", "Spring Bank Holiday"),
            ("2022-06-03", "Queen's Platinum Jubilee Bank Holiday"),
            ("2022-08-29", "Summer Bank Holiday"),
            ("2022-09-19", "State Funeral of Queen Elizabeth II"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (substitute day)"),
        )

    def test_2023(self):
        # https://www.gov.gg/holidaydates
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (substitute day)"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "May Day Bank Holiday"),
            ("2023-05-08", "Extra Public Holiday for the Coronation of King Charles III"),
            ("2023-05-09", "Liberation Day"),
            ("2023-05-29", "Spring Bank Holiday"),
            ("2023-08-28", "Summer Bank Holiday"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        # https://www.gov.gg/holidaydates
        # https://www.bbc.co.uk/news/articles/c1441ddn87po
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Day Bank Holiday"),
            ("2024-05-09", "Liberation Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-07-16", "The visit of His Majesty King Charles III and Queen Camilla"),
            ("2024-08-26", "Summer Bank Holiday"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_2025(self):
        # https://www.gov.gg/holidaydates
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-05", "May Day Bank Holiday"),
            ("2025-05-09", "Liberation Day"),
            ("2025-05-26", "Spring Bank Holiday"),
            ("2025-08-25", "Summer Bank Holiday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars import (
    _CustomBuddhistHolidays,
    _CustomChineseHolidays,
    _CustomHinduHolidays,
    _CustomIslamicHolidays,
)
from holidays.calendars.gregorian import (
    JAN,
    FEB,
    MAR,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    NOV,
    DEC,
    FRI,
    SAT,
    SUN,
)
from holidays.groups import (
    BuddhistCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    FRI_TO_NEXT_WORKDAY,
    SAT_TO_NEXT_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
)

if TYPE_CHECKING:
    from datetime import date


class Malaysia(
    ObservedHolidayBase,
    BuddhistCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Malaysia holidays.

    References:
        * [Holidays Act 1951](https://web.archive.org/web/20241202103403/https://www.kabinet.gov.my/bkpp/pdf/akta_warta/1951_12_31_act369.pdf)
        * [Holidays Ordinance (Sabah Cap. 56)](https://web.archive.org/web/20201028045259/https://sagc.sabah.gov.my/sites/default/files/law/HolidaysOrdinance.pdf)
        * [Public Holidays Ordinance (Sarawak Cap. 8)](https://web.archive.org/web/20221208142318/https://www.kabinet.gov.my/bkpp/pdf/akta_warta/sarawak_public_holidays_ord_chapter8.pdf)
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Malaysia>
        * <https://web.archive.org/web/20250123115300/https://www.nst.com.my/news/nation/2020/03/571660/agongs-birthday-moved-june-6-june-8>
        * <https://web.archive.org/web/20240228225038/https://www.nst.com.my/news/nation/2024/02/1014012/melaka-cm-suggests-declaring-feb-20-federal-public-holiday-mark>
        * <https://web.archive.org/web/20251216120554/https://www.kabinet.gov.my/storage/2025/08/HKA-2026.pdf>

    Subdivisions Holidays References:
        * Sabah:
            * [2001](https://web.archive.org/web/20230605164302/https://www.sabah.gov.my/gazette/docs/000207.pdf)
            * [2002](https://web.archive.org/web/20230605173234/https://www.sabah.gov.my/gazette/docs/000968.pdf)
            * [2003](https://web.archive.org/web/20230418041718/https://www.sabah.gov.my/gazette/docs/001736.pdf)
            * [2004](https://web.archive.org/web/20230414022317/https://www.sabah.gov.my/gazette/docs/001546.pdf)
            * [2005](https://web.archive.org/web/20230420233014/https://www.sabah.gov.my/gazette/docs/001714.pdf)
            * [2006](https://web.archive.org/web/20230414141628/https://www.sabah.gov.my/gazette/docs/001803.pdf)
            * [2007](https://web.archive.org/web/20230414125605/https://www.sabah.gov.my/gazette/docs/001866.pdf)
            * [2008](https://web.archive.org/web/20230605175854/https://www.sabah.gov.my/gazette/docs/001942.pdf)
            * [2009](https://web.archive.org/web/20230605180330/https://www.sabah.gov.my/gazette/docs/001998.pdf)
            * [Christmas Eve since 2019](https://web.archive.org/web/20221207071317/https://www.sabah.gov.my/gazette/docs/002712.pdf)
            * [Birthday of the Governor of Sabah update 2010](https://web.archive.org/web/20230414125515/https://www.sabah.gov.my/gazette/docs/002114.pdf)
            * [2025](https://web.archive.org/web/20251216152001/https://www.sabah.gov.my/gazette/docs/003106.pdf)
            * [2025 changes](https://web.archive.org/web/20250330061412/https://www.sabah.gov.my/gazette/docs/003144.pdf)
            * [2026](https://web.archive.org/web/20251010011859/https://www.sabah.gov.my/gazette/docs/003152.pdf)

    Section 3 of Holidays Act 1951:
    > If any day specified in the Schedule falls on Sunday then the day following shall be
    > a public holiday and if such day is already a public holiday, then the day following
    > shall be a public holiday".

    In Johor (until 1994 and in 2014-2024) and Kedah it's Friday to Sunday,
    in Kelantan and Terengganu - Saturday to Sunday.
    """

    country = "MY"
    default_language = "ms_MY"
    # %s (estimated).
    estimated_label = tr("%s (anggaran)")
    # %s (in lieu).
    observed_label = tr("Cuti %s")
    # %s (observed, estimated).
    observed_estimated_label = tr("Cuti %s (anggaran)")
    start_year = 1952
    subdivisions = (
        "01",  # Johor.
        "02",  # Kedah.
        "03",  # Kelantan.
        "04",  # Melaka.
        "05",  # Negeri Sembilan.
        "06",  # Pahang.
        "07",  # Pulau Pinang.
        "08",  # Perak.
        "09",  # Perlis.
        "10",  # Selangor.
        "11",  # Terengganu.
        "12",  # Sabah.
        "13",  # Sarawak.
        "14",  # Wilayah Persekutuan Kuala Lumpur.
        "15",  # Wilayah Persekutuan Labuan.
        "16",  # Wilayah Persekutuan Putrajaya.
    )
    subdivisions_aliases = {
        "Johor": "01",
        "JHR": "01",
        "Kedah": "02",
        "KDH": "02",
        "Kelantan": "03",
        "KTN": "03",
        "Melaka": "04",
        "MLK": "04",
        "Negeri Sembilan": "05",
        "NSN": "05",
        "Pahang": "06",
        "PHG": "06",
        "Pulau Pinang": "07",
        "PNG": "07",
        "Perak": "08",
        "PRK": "08",
        "Perlis": "09",
        "PLS": "09",
        "Selangor": "10",
        "SGR": "10",
        "Terengganu": "11",
        "TRG": "11",
        "Sabah": "12",
        "SBH": "12",
        "Sarawak": "13",
        "SWK": "13",
        "Wilayah Persekutuan Kuala Lumpur": "14",
        "KUL": "14",
        "Wilayah Persekutuan Labuan": "15",
        "LBN": "15",
        "Wilayah Persekutuan Putrajaya": "16",
        "PJY": "16",
    }
    supported_languages = ("en_US", "ms_MY", "th")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        BuddhistCalendarHolidays.__init__(self, cls=MalaysiaBuddhistHolidays, show_estimated=True)
        ChineseCalendarHolidays.__init__(self, cls=MalaysiaChineseHolidays, show_estimated=True)
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=MalaysiaHinduHolidays)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=MalaysiaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, cls=MalaysiaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_WORKDAY)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        if (
            self.subdiv == "01" and (dt.year <= 1994 or 2014 <= dt.year <= 2024)
        ) or self.subdiv in {"02", "03", "11"}:
            return {FRI, SAT}
        else:
            return {SAT, SUN}

    def _populate_public_holidays(self):
        self.dts_observed = set()

        # Chinese New Year.
        self.dts_observed.add(self._add_chinese_new_years_day(tr("Tahun Baharu Cina")))

        self.dts_observed.add(
            # Chinese New Year (Second Day).
            self._add_chinese_new_years_day_two(tr("Tahun Baharu Cina (Hari Kedua)"))
        )

        # Vesak Day.
        self.dts_observed.add(self._add_vesak_may(tr("Hari Wesak")))

        if self._year >= 1973:
            # Labor Day.
            self.dts_observed.add(self._add_labor_day(tr("Hari Pekerja")))

        # Birthday of HM Yang di-Pertuan Agong.
        name = tr("Hari Keputeraan Rasmi Seri Paduka Baginda Yang di-Pertuan Agong")
        if self._year <= 2016:
            self.dts_observed.add(self._add_holiday_1st_sat_of_jun(name))
        elif self._year <= 2019:
            self.dts_observed.add(self._add_holiday_sep_9(name))
        elif self._year == 2020:
            self.dts_observed.add(self._add_holiday_jun_8(name))
        else:
            self.dts_observed.add(self._add_holiday_1st_mon_of_jun(name))

        # National Day.
        self.dts_observed.add(self._add_holiday_aug_31(tr("Hari Kebangsaan")))

        if self._year >= 2010:
            # Malaysia Day.
            self.dts_observed.add(self._add_holiday_sep_16(tr("Hari Malaysia")))

        # Christmas Day.
        self.dts_observed.add(self._add_christmas_day(tr("Hari Krismas")))

        if self._year >= 1995:
            # Islamic New Year.
            self._add_islamic_new_year_day(tr("Awal Muharam"))

        # Prophet Muhammad's Birthday.
        self.dts_observed.update(self._add_mawlid_day(tr("Hari Keputeraan Nabi Muhammad S.A.W.")))

        # Eid al-Fitr.
        self.dts_observed.update(self._add_eid_al_fitr_day(tr("Hari Raya Puasa")))

        # Eid al-Fitr (Second Day).
        self.dts_observed.update(self._add_eid_al_fitr_day_two(tr("Hari Raya Puasa (Hari Kedua)")))

        # Eid al-Adha.
        self.dts_observed.update(self._add_eid_al_adha_day(tr("Hari Raya Qurban")))

    def _populate_subdiv_holidays(self):
        if self.subdiv and self.subdiv not in {"13", "15"}:
            # Deepavali.
            self.dts_observed.add(self._add_diwali(tr("Hari Deepavali")))

        super()._populate_subdiv_holidays()

        if (
            self.subdiv == "01" and (self._year <= 1994 or 2014 <= self._year <= 2024)
        ) or self.subdiv == "02":
            self._observed_rule = FRI_TO_NEXT_WORKDAY
        elif self.subdiv in {"03", "11"}:
            self._observed_rule = SAT_TO_NEXT_WORKDAY
        else:
            self._observed_rule = SUN_TO_NEXT_WORKDAY

        if self.observed:
            self._populate_observed(self.dts_observed)

    def _populate_subdiv_01_public_holidays(self):
        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 2015:
            # Birthday of the Sultan of Johor.
            self._add_holiday_mar_23(tr("Hari Keputeraan Sultan Johor"))

        if self._year >= 2011:
            # The Sultan of Johor Hol.
            self._add_hari_hol_johor(tr("Hari Hol Almarhum Sultan Iskandar"))

        # Beginning of Ramadan.
        self.dts_observed.update(self._add_ramadan_beginning_day(tr("Awal Ramadan")))

    def _populate_subdiv_02_public_holidays(self):
        if self._year >= 2022:
            # Thaipusam.
            self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 2018:
            # Birthday of The Sultan of Kedah.
            name = tr("Hari Keputeraan Sultan Kedah")
            if self._year == 2024:
                self._add_holiday_jun_30(name)
            else:
                self._add_holiday_3rd_sun_of_jun(name)

        # Isra' and Mi'raj.
        self.dts_observed.update(self._add_isra_and_miraj_day(tr("Israk dan Mikraj")))

        # Beginning of Ramadan.
        self.dts_observed.update(self._add_ramadan_beginning_day(tr("Awal Ramadan")))

        # Eid al-Adha (Second Day).
        self.dts_observed.update(
            self._add_eid_al_adha_day_two(tr("Hari Raya Qurban (Hari Kedua)"))
        )

    def _populate_subdiv_03_public_holidays(self):
        if self._year >= 2010:
            # Birthday of the Sultan of Kelantan.
            name = tr("Hari Keputeraan Sultan Kelantan")
            if self._year >= 2023:
                self._add_holiday_sep_29(name)
                self._add_holiday_sep_30(name)
            elif self._year >= 2012:
                self._add_holiday_nov_11(name)
                self._add_holiday_nov_12(name)
            else:
                self._add_holiday_mar_30(name)
                self._add_holiday_mar_31(name)

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

        if self._year >= 2023:
            # Arafat Day.
            self.dts_observed.update(self._add_arafah_day(tr("Hari Arafah")))

        # Eid al-Adha (Second Day).
        self.dts_observed.update(
            self._add_eid_al_adha_day_two(tr("Hari Raya Qurban (Hari Kedua)"))
        )

    def _populate_subdiv_04_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        if self._year >= 2024:
            self.dts_observed.add(
                # Declaration of Independence Day.
                self._add_holiday_feb_20(tr("Hari Pengisytiharan Tarikh Kemerdekaan"))
            )
        elif self._year >= 1989:
            self.dts_observed.add(
                self._add_holiday_apr_15(
                    # Declaration of Malacca as a Historical City.
                    tr("Hari Perisytiharan Melaka Sebagai Bandaraya Bersejarah")
                )
            )

        # Birthday of the Governor of Malacca.
        name = tr("Hari Jadi Yang di-Pertua Negeri Melaka")
        self.dts_observed.add(
            self._add_holiday_aug_24(name)
            if self._year >= 2020
            else self._add_holiday_2nd_fri_of_oct(name)
        )

        if self._year >= 2025:
            self.dts_observed.update(
                # Eid al-Fitr (Third Day).
                self._add_eid_al_fitr_day_three(tr("Hari Raya Puasa (Hari Ketiga)"))
            )
        else:
            # Beginning of Ramadan.
            self.dts_observed.update(self._add_ramadan_beginning_day(tr("Awal Ramadan")))

    def _populate_subdiv_05_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 2009:
            self.dts_observed.add(
                self._add_holiday_jan_14(
                    # Birthday of the Sultan of Negeri Sembilan.
                    tr("Hari Keputeraan Yang di-Pertuan Besar Negeri Sembilan")
                )
            )

        # Isra' and Mi'raj.
        self.dts_observed.update(self._add_isra_and_miraj_day(tr("Israk dan Mikraj")))

    def _populate_subdiv_06_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        if self._year >= 1975:
            # The Sultan of Pahang Hol.
            name = tr("Hari Hol Sultan Pahang")
            self.dts_observed.add(
                self._add_holiday_may_22(name)
                if self._year >= 2020
                else self._add_holiday_may_7(name)
            )

            # Birthday of the Sultan of Pahang.
            name = tr("Hari Keputeraan Sultan Pahang")
            self.dts_observed.add(
                self._add_holiday_jul_30(name)
                if self._year >= 2019
                else self._add_holiday_oct_24(name)
            )

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_07_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 2009:
            self.dts_observed.add(
                # George Town Heritage Day.
                self._add_holiday_jul_7(tr("Hari Ulang Tahun Perisytiharan Tapak Warisan Dunia"))
            )

        self.dts_observed.add(
            # Birthday of the Governor of Penang.
            self._add_holiday_2nd_sat_of_jul(tr("Hari Jadi Yang di-Pertua Negeri Pulau Pinang"))
        )

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_08_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        # Birthday of the Sultan of Perak.
        name = tr("Hari Keputeraan Sultan Perak")
        if self._year >= 2018:
            self._add_holiday_1st_fri_of_nov(name)
        else:
            self._add_holiday_nov_27(name)

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_09_public_holidays(self):
        if self._year >= 2000:
            # Birthday of the Raja of Perlis.
            name = tr("Hari Ulang Tahun Keputeraan Raja Perlis")
            self.dts_observed.add(
                self._add_holiday_jul_17(name)
                if 2018 <= self._year <= 2021
                else self._add_holiday_may_17(name)
            )

        # Isra' and Mi'raj.
        self.dts_observed.update(self._add_isra_and_miraj_day(tr("Israk dan Mikraj")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

        # Eid al-Adha (Second Day).
        self.dts_observed.update(
            self._add_eid_al_adha_day_two(tr("Hari Raya Qurban (Hari Kedua)"))
        )

    def _populate_subdiv_10_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        # Birthday of The Sultan of Selangor.
        self.dts_observed.add(self._add_holiday_dec_11(tr("Hari Keputeraan Sultan Selangor")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_11_public_holidays(self):
        if self._year >= 2000:
            self.dts_observed.add(
                self._add_holiday_mar_4(
                    # Anniversary of the Installation of the Sultan of Terengganu.
                    tr("Hari Ulang Tahun Pertabalan Sultan Terengganu")
                )
            )

            self.dts_observed.add(
                # Birthday of the Sultan of Terengganu.
                self._add_holiday_apr_26(tr("Hari Keputeraan Sultan Terengganu"))
            )

        if self._year >= 2020:
            # Isra' and Mi'raj.
            self.dts_observed.update(self._add_isra_and_miraj_day(tr("Israk dan Mikraj")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

        # Arafat Day.
        self.dts_observed.update(self._add_arafah_day(tr("Hari Arafah")))

        self.dts_observed.update(
            # Eid al-Adha (Second Day).
            self._add_eid_al_adha_day_two(tr("Hari Raya Qurban (Hari Kedua)"))
        )

    def _populate_subdiv_12_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Pesta Kaamatan.
        name = tr("Pesta Kaamatan")
        self._add_holiday_may_30(name)
        self._add_holiday_may_31(name)

        if self._year >= 2001:
            # Birthday of the Governor of Sabah.
            name = tr("Hari Jadi Yang di-Pertua Negeri Sabah")
            if self._year >= 2025:
                self._add_holiday_mar_30(name)
            elif self._year >= 2010:
                self._add_holiday_1st_sat_of_oct(name)
            elif self._year == 2005:
                self._add_holiday_feb_11(name)
            else:
                self._add_holiday_sep_16(name)

        if self._year >= 2019:
            # Christmas Eve.
            self._add_christmas_eve(tr("Krismas (Eve)"))

    def _populate_subdiv_13_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        if self._year >= 1965:
            # Dayak Festival Day.
            name = tr("Perayaan Hari Gawai Dayak")
            self.dts_observed.add(self._add_holiday_jun_1(name))
            self.dts_observed.add(self._add_holiday_jun_2(name))

        # Birthday of the Governor of Sarawak.
        self._add_holiday_2nd_sat_of_oct(tr("Hari Jadi Yang di-Pertua Negeri Sarawak"))

        if self._year >= 2017:
            # Sarawak Independence Day.
            self.dts_observed.add(self._add_holiday_jul_22(tr("Hari Kemerdekaan Sarawak")))

    def _populate_subdiv_14_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 1974:
            # Federal Territory Day.
            self.dts_observed.add(self._add_holiday_feb_1(tr("Hari Wilayah Persekutuan")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_15_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        if self._year >= 1974:
            # Federal Territory Day.
            self.dts_observed.add(self._add_holiday_feb_1(tr("Hari Wilayah Persekutuan")))

        # Pesta Kaamatan.
        name = tr("Pesta Kaamatan")
        self._add_holiday_may_30(name)
        self._add_holiday_may_31(name)

        if self._year >= 2014:
            # Deepavali.
            self.dts_observed.add(self._add_diwali(tr("Hari Deepavali")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))

    def _populate_subdiv_16_public_holidays(self):
        # New Year's Day.
        self.dts_observed.add(self._add_new_years_day(tr("Tahun Baharu")))

        # Thaipusam.
        self.dts_observed.add(self._add_thaipusam(tr("Hari Thaipusam")))

        if self._year >= 1974:
            # Federal Territory Day.
            self.dts_observed.add(self._add_holiday_feb_1(tr("Hari Wilayah Persekutuan")))

        # Nuzul Al-Quran Day.
        self.dts_observed.update(self._add_nuzul_al_quran_day(tr("Hari Nuzul Al-Quran")))


class MY(Malaysia):
    pass


class MYS(Malaysia):
    pass


class MalaysiaBuddhistHolidays(_CustomBuddhistHolidays):
    VESAK_MAY_DATES = {
        2001: (MAY, 7),
        2002: (MAY, 27),
        2003: (MAY, 15),
        2004: (MAY, 3),
        2005: (MAY, 22),
        2006: (MAY, 12),
        2007: (MAY, 1),
        2008: (MAY, 19),
        2009: (MAY, 9),
        2010: (MAY, 28),
        2011: (MAY, 17),
        2012: (MAY, 5),
        2013: (MAY, 24),
        2014: (MAY, 13),
        2015: (MAY, 3),
        2016: (MAY, 21),
        2017: (MAY, 10),
        2018: (MAY, 29),
        2019: (MAY, 19),
        2020: (MAY, 7),
        2021: (MAY, 26),
        2022: (MAY, 15),
        2023: (MAY, 4),
        2024: (MAY, 22),
        2025: (MAY, 12),
        2026: (MAY, 31),
    }


class MalaysiaChineseHolidays(_CustomChineseHolidays):
    LUNAR_NEW_YEAR_DATES_CONFIRMED_YEARS = (2001, 2026)


class MalaysiaHinduHolidays(_CustomHinduHolidays):
    DIWALI_DATES = {
        2001: (NOV, 14),
        2002: (NOV, 3),
        2003: (OCT, 23),
        2004: (NOV, 11),
        2005: (NOV, 1),
        2006: (OCT, 21),
        2007: (NOV, 8),
        2008: (OCT, 27),
        2009: (OCT, 17),
        2010: (NOV, 5),
        2011: (OCT, 26),
        2012: (NOV, 13),
        2013: (NOV, 2),
        2014: (OCT, 22),
        2015: (NOV, 10),
        2016: (OCT, 29),
        2017: (OCT, 18),
        2018: (NOV, 6),
        2019: (OCT, 27),
        2020: (NOV, 14),
        2021: (NOV, 4),
        2022: (OCT, 24),
        2023: (NOV, 12),
        2024: (OCT, 31),
        2025: (OCT, 20),
        2026: (NOV, 8),
    }

    THAIPUSAM_DATES = {
        2011: (JAN, 20),
        2012: (FEB, 7),
        2013: (JAN, 27),
        2014: (JAN, 17),
        2015: (FEB, 3),
        2016: (JAN, 24),
        2017: (FEB, 9),
        2018: (JAN, 31),
        2019: (JAN, 21),
        2020: (FEB, 8),
        2021: (JAN, 28),
        2022: (JAN, 18),
        2023: (FEB, 5),
        2024: (JAN, 25),
        2025: (FEB, 11),
        2026: (FEB, 1),
        2027: (JAN, 22),
    }


class MalaysiaIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2026)
    EID_AL_ADHA_DATES = {
        2001: (MAR, 6),
        2002: (FEB, 23),
        2003: (FEB, 12),
        2004: (FEB, 2),
        2008: (DEC, 9),
        2009: (NOV, 28),
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 12),
        2018: (AUG, 22),
        2022: (JUL, 10),
        2023: (JUN, 29),
        2024: (JUN, 17),
        2025: (JUN, 7),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2026)
    EID_AL_FITR_DATES = {
        2001: (DEC, 17),
        2002: (DEC, 6),
        2003: (NOV, 26),
        2006: (OCT, 24),
        2011: (AUG, 31),
        2019: (JUN, 5),
        2023: (APR, 22),
        2025: (MAR, 31),
        2026: (MAR, 21),
    }

    HARI_HOL_JOHOR_DATES_CONFIRMED_YEARS = (2011, 2026)
    HARI_HOL_JOHOR_DATES = {
        2011: (JAN, 12),
        2012: (DEC, 20),
        2013: (DEC, 10),
        2014: (NOV, 29),
        2015: (NOV, 19),
        2016: (NOV, 7),
        2017: (OCT, 27),
        2020: (SEP, 24),
        2022: (SEP, 3),
        2023: (AUG, 23),
        2024: (AUG, 11),
        2026: (JUL, 21),
    }

    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2001, 2026)
    HIJRI_NEW_YEAR_DATES = {
        2003: (MAR, 5),
        2004: (FEB, 22),
        2010: (DEC, 8),
        2011: (NOV, 27),
        2013: (NOV, 5),
        2017: (SEP, 22),
        2019: (SEP, 1),
        2021: (AUG, 10),
        2025: (JUN, 27),
        2026: (JUN, 17),
    }

    ISRA_AND_MIRAJ_DATES_CONFIRMED_YEARS = (2001, 2026)
    ISRA_AND_MIRAJ_DATES = {
        2001: (OCT, 15),
        2006: (AUG, 22),
        2007: (AUG, 11),
        2008: (JUL, 31),
        2014: (MAY, 27),
        2016: (MAY, 5),
        2018: (APR, 14),
        2022: (MAR, 1),
        2026: (JAN, 17),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2001, 2026)
    MAWLID_DATES = {
        2003: (MAY, 14),
        2004: (MAY, 2),
        2006: (APR, 11),
        2011: (FEB, 16),
        2012: (FEB, 5),
        2014: (JAN, 14),
        2015: ((JAN, 3), (DEC, 24)),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2021: (OCT, 19),
        2022: (OCT, 10),
        2023: (SEP, 28),
        2024: (SEP, 16),
        2025: (SEP, 5),
    }

    NUZUL_AL_QURAN_DATES_CONFIRMED_YEARS = (2001, 2026)
    NUZUL_AL_QURAN_DATES = {
        2001: (DEC, 3),
        2003: (NOV, 12),
        2004: (NOV, 1),
        2005: (OCT, 21),
        2008: (SEP, 18),
        2014: (JUL, 15),
        2018: (JUN, 2),
        2022: (APR, 19),
        2024: (MAR, 28),
        2025: (MAR, 18),
        2026: (MAR, 7),
    }

    RAMADAN_BEGINNING_DATES_CONFIRMED_YEARS = (2001, 2026)
    RAMADAN_BEGINNING_DATES = {
        2001: (NOV, 17),
        2003: (OCT, 27),
        2004: (OCT, 16),
        2005: (OCT, 5),
        2008: (SEP, 2),
        2014: (JUN, 29),
        2016: (JUN, 7),
        2018: (MAY, 17),
        2022: (APR, 3),
        2024: (MAR, 12),
        2025: (MAR, 2),
        2026: (FEB, 19),
    }


class MalaysiaStaticHolidays:
    # General election additional holiday.
    general_election_additional_holiday = tr("Cuti Peristiwa (pilihan raya umum)")

    # Additional holiday.
    additional_holiday = tr("Cuti Peristiwa")

    # Eid al-Adha.
    eid_al_adha = tr("Hari Raya Qurban")

    # Labor Day.
    labor_day = tr("Hari Pekerja")

    # Malaysia Cup Holiday.
    malaysia_cup_holiday = tr("Cuti Piala Malaysia")

    special_public_holidays = {
        1999: (NOV, 29, general_election_additional_holiday),
        2017: (
            # Day of Installation of the 15th Yang di-Pertuan Agong.
            (APR, 24, tr("Hari Pertabalan Yang di-Pertuan Agong ke-15")),
            # Additional holiday in commemoration of the 2017 SEA Games.
            (SEP, 4, tr("Cuti tambahan sempena memperingati SAT 2017")),
        ),
        2018: (MAY, 9, general_election_additional_holiday),
        # Day of Installation of the 16th Yang di-Pertuan Agong.
        2019: (JUL, 30, tr("Hari Pertabalan Yang di-Pertuan Agong ke-16")),
        2022: (
            (NOV, 18, general_election_additional_holiday),
            (NOV, 19, general_election_additional_holiday),
            (NOV, 28, additional_holiday),
        ),
        # Eid al-Fitr (additional holiday).
        2023: (APR, 21, tr("Hari Raya Puasa (pergantian hari)")),
        2025: (SEP, 15, additional_holiday),
    }

    special_01_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 13, additional_holiday),
        ),
    }
    special_01_public_holidays_observed = {
        2022: (MAY, 4, labor_day),
    }

    special_02_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 13, additional_holiday),
        ),
    }
    special_02_public_holidays_observed = {
        2022: (MAY, 4, labor_day),
    }

    special_03_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 13, additional_holiday),
        ),
    }
    special_03_public_holidays_observed = {
        2022: (MAY, 4, labor_day),
    }

    special_04_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_04_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_05_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_05_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_06_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_06_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_07_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_07_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_08_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_08_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_09_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_09_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_10_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
    }
    special_10_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_11_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 13, additional_holiday),
        ),
    }
    special_11_public_holidays_observed = {
        # Arafat Day.
        2007: (JAN, 2, tr("Hari Arafah")),
        2022: (MAY, 4, labor_day),
    }

    special_12_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_13_public_holidays = {
        2018: (
            (MAY, 17, additional_holiday),
            (MAY, 18, additional_holiday),
        ),
    }
    special_13_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_14_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
        2021: (DEC, 3, malaysia_cup_holiday),
    }
    special_14_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_15_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
        2021: (DEC, 3, malaysia_cup_holiday),
    }
    special_15_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }

    special_16_public_holidays = {
        2018: (
            (MAY, 10, additional_holiday),
            (MAY, 11, additional_holiday),
        ),
        2021: (DEC, 3, malaysia_cup_holiday),
    }
    special_16_public_holidays_observed = {
        2007: (JAN, 2, eid_al_adha),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.calendars.gregorian import _timedelta
from holidays.countries.turkey import Turkey
from tests.common import CommonCountryTests


class TestTurkey(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Turkey)

    def test_special_holidays(self):
        self.assertHoliday("1999-12-31")

    def test_new_years_day(self):
        self.assertHolidayName("Yılbaşı", (f"{year}-01-01" for year in self.full_range))

    def test_national_sovereignty_and_childrens_day(self):
        name_1936 = "Ulusal Egemenlik Bayramı"
        name_1981 = "Ulusal Egemenlik ve Çocuk Bayramı"
        self.assertHolidayName(
            name_1936, (f"{year}-04-23" for year in range(self.start_year, 1981))
        )
        self.assertHolidayName(name_1981, (f"{year}-04-23" for year in range(1981, self.end_year)))
        self.assertNoHolidayName(name_1936, range(1981, self.end_year))
        self.assertNoHolidayName(name_1981, range(self.start_year, 1981))

    def test_spring_and_labor_day(self):
        name_1936 = "Bahar Bayramı"
        name_2009 = "Emek ve Dayanışma Günü"
        self.assertHolidayName(
            name_1936, (f"{year}-05-01" for year in range(self.start_year, 1981))
        )
        self.assertHolidayName(name_2009, (f"{year}-05-01" for year in range(2009, self.end_year)))
        self.assertNoHolidayName(name_1936, range(1981, self.end_year))
        self.assertNoHolidayName(name_2009, range(self.start_year, 2009))

    def test_ataturk_youth_and_sports_day(self):
        name_1936 = "Gençlik ve Spor Bayramı"
        name_1981 = "Atatürk'ü Anma, Gençlik ve Spor Bayramı"
        self.assertHolidayName(
            name_1936, (f"{year}-05-19" for year in range(self.start_year, 1981))
        )
        self.assertHolidayName(name_1981, (f"{year}-05-19" for year in range(1981, self.end_year)))
        self.assertNoHolidayName(name_1936, range(1981, self.end_year))
        self.assertNoHolidayName(name_1981, range(self.start_year, 1981))

    def test_freedom_and_constitution_day(self):
        name = "Hürriyet ve Anayasa Bayramı"
        self.assertHolidayName(name, (f"{year}-05-27" for year in range(1963, 1981)))
        self.assertNoHolidayName(name, range(self.start_year, 1963), range(1981, self.end_year))

    def test_democracy_and_national_unity_day(self):
        name = "Demokrasi ve Millî Birlik Günü"
        self.assertHolidayName(name, (f"{year}-07-15" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_victory_day(self):
        self.assertHolidayName("Zafer Bayramı", (f"{year}-08-30" for year in self.full_range))

    def test_republic_day(self):
        name = "Cumhuriyet Bayramı"
        self.assertHolidayName(
            name,
            (f"{year}-10-29" for year in self.full_range),
            (f"{year}-10-30" for year in range(self.start_year, 1981)),
        )
        self.assertHalfDayHolidayName(
            f"{name} (saat 13.00'ten)",
            (f"{year}-10-28" for year in self.full_range),
        )

    def test_eid_al_fitr(self):
        name = "Ramazan Bayramı"
        for ymd in (
            (2000, 1, 8),
            (2000, 12, 27),
            (2010, 9, 9),
            (2018, 6, 15),
            (2019, 6, 4),
            (2020, 5, 24),
            (2021, 5, 13),
            (2022, 5, 2),
            (2023, 4, 21),
        ):
            dt = date(*ymd)
            self.assertHolidayName(name, dt, _timedelta(dt, +1), _timedelta(dt, +2))
            self.assertHalfDayHolidayName(f"{name} (saat 13.00'ten)", _timedelta(dt, -1))
        exception_years = {1968, 2000, 2033}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, (year for year in self.full_range if year not in exception_years)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, exception_years)

    def test_eid_al_adha(self):
        name = "Kurban Bayramı"
        for ymd in (
            (2006, 1, 10),
            (2006, 12, 31),
            (2010, 11, 16),
            (2018, 8, 21),
            (2019, 8, 11),
            (2020, 7, 31),
            (2021, 7, 20),
            (2022, 7, 9),
            (2023, 6, 28),
        ):
            dt = date(*ymd)
            self.assertHolidayName(
                name, dt, _timedelta(dt, +1), _timedelta(dt, +2), _timedelta(dt, +3)
            )
            self.assertHalfDayHolidayName(f"{name} (saat 13.00'ten)", _timedelta(dt, -1))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name,
            4,
            (year for year in self.full_range if year not in {1941, 1942, 1974, 2006, 2007, 2039}),
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 5, 1942, 2006)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 7, 1941, 2007)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 8, 1974, 2039)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Yılbaşı"),
            ("2022-04-23", "Ulusal Egemenlik ve Çocuk Bayramı"),
            ("2022-05-01", "Emek ve Dayanışma Günü"),
            ("2022-05-02", "Ramazan Bayramı"),
            ("2022-05-03", "Ramazan Bayramı"),
            ("2022-05-04", "Ramazan Bayramı"),
            ("2022-05-19", "Atatürk'ü Anma, Gençlik ve Spor Bayramı"),
            ("2022-07-09", "Kurban Bayramı"),
            ("2022-07-10", "Kurban Bayramı"),
            ("2022-07-11", "Kurban Bayramı"),
            ("2022-07-12", "Kurban Bayramı"),
            ("2022-07-15", "Demokrasi ve Millî Birlik Günü"),
            ("2022-08-30", "Zafer Bayramı"),
            ("2022-10-29", "Cumhuriyet Bayramı"),
        )

    def test_2022_half_day(self):
        self.assertHalfDayHolidaysInYear(
            2022,
            ("2022-05-01", "Ramazan Bayramı (saat 13.00'ten)"),
            ("2022-07-08", "Kurban Bayramı (saat 13.00'ten)"),
            ("2022-10-28", "Cumhuriyet Bayramı (saat 13.00'ten)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Yılbaşı"),
            ("2023-04-20", "Ramazan Bayramı (saat 13.00'ten)"),
            ("2023-04-21", "Ramazan Bayramı"),
            ("2023-04-22", "Ramazan Bayramı"),
            ("2023-04-23", "Ramazan Bayramı; Ulusal Egemenlik ve Çocuk Bayramı"),
            ("2023-05-01", "Emek ve Dayanışma Günü"),
            ("2023-05-19", "Atatürk'ü Anma, Gençlik ve Spor Bayramı"),
            ("2023-06-27", "Kurban Bayramı (saat 13.00'ten)"),
            ("2023-06-28", "Kurban Bayramı"),
            ("2023-06-29", "Kurban Bayramı"),
            ("2023-06-30", "Kurban Bayramı"),
            ("2023-07-01", "Kurban Bayramı"),
            ("2023-07-15", "Demokrasi ve Millî Birlik Günü"),
            ("2023-08-30", "Zafer Bayramı"),
            ("2023-10-28", "Cumhuriyet Bayramı (saat 13.00'ten)"),
            ("2023-10-29", "Cumhuriyet Bayramı"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-04-20", "Eid al-Fitr (from 1pm)"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr; National Sovereignty and Children's Day"),
            ("2023-05-01", "Labour and Solidarity Day"),
            ("2023-05-19", "Commemoration of Atatürk, Youth and Sports Day"),
            ("2023-06-27", "Eid al-Adha (from 1pm)"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Eid al-Adha"),
            ("2023-07-01", "Eid al-Adha"),
            ("2023-07-15", "Democracy and National Unity Day"),
            ("2023-08-30", "Victory Day"),
            ("2023-10-28", "Republic Day (from 1pm)"),
            ("2023-10-29", "Republic Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-04-20", "Рамазан-байрам (з 13:00)"),
            ("2023-04-21", "Рамазан-байрам"),
            ("2023-04-22", "Рамазан-байрам"),
            ("2023-04-23", "День національної незалежності та дітей; Рамазан-байрам"),
            ("2023-05-01", "День праці та солідарності"),
            ("2023-05-19", "День вшанування памʼяті Ататюрка, молоді та спорту"),
            ("2023-06-27", "Курбан-байрам (з 13:00)"),
            ("2023-06-28", "Курбан-байрам"),
            ("2023-06-29", "Курбан-байрам"),
            ("2023-06-30", "Курбан-байрам"),
            ("2023-07-01", "Курбан-байрам"),
            ("2023-07-15", "День демократії та національної єдності"),
            ("2023-08-30", "День Перемоги"),
            ("2023-10-28", "День Республіки (з 13:00)"),
            ("2023-10-29", "День Республіки"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.venezuela import Venezuela
from tests.common import CommonCountryTests


class TestVenezuela(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Venezuela)

    def test_special_holidays(self):
        self.assertHoliday(
            "2009-02-02",
            "2010-03-29",
            "2010-03-30",
            "2010-03-31",
            "2013-12-08",
            "2014-02-27",
            "2014-02-28",
            "2016-03-21",
            "2016-03-22",
            "2016-03-23",
            "2016-04-08",
            "2016-04-15",
            "2016-04-18",
            "2016-04-22",
            "2016-04-29",
            "2016-05-06",
            "2016-05-13",
            "2016-05-20",
            "2016-05-27",
            "2017-04-10",
            "2017-04-11",
            "2017-04-12",
            "2018-08-20",
            "2019-02-28",
            "2019-03-01",
            "2019-03-11",
            "2019-03-12",
            "2019-03-26",
            "2025-10-19",
            "2025-10-20",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_carnival_monday(self):
        name = "Lunes de Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Martes de Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_declaration_of_independence(self):
        self.assertHolidayName(
            "Declaración de la Independencia", (f"{year}-04-19" for year in self.full_range)
        )

    def test_international_workers_day(self):
        name = "Dia Mundial del Trabajador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1945, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1945))

    def test_family_day(self):
        name = "Día de la Familia"
        self.assertHolidayName(name, (f"{year}-05-15" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_battle_of_carabobo(self):
        name = "Batalla de Carabobo"
        self.assertHolidayName(
            name,
            (
                f"{year}-06-24"
                for year in (*range(self.start_year, 1918), *range(1971, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1918, 1971))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-07-05" for year in self.full_range)
        )

    def test_birthday_of_simon_bolivar(self):
        name = "Natalicio de Simón Bolívar"
        self.assertHolidayName(name, (f"{year}-07-24" for year in range(1918, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1918))

    def test_day_of_indigeneous_resistance(self):
        name_1921 = "Día de la Raza"
        name_2002 = "Día de la Resistencia Indígena"
        self.assertHolidayName(name_1921, (f"{year}-10-12" for year in range(1921, 2002)))
        self.assertHolidayName(name_2002, (f"{year}-10-12" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(
            name_1921, range(self.start_year, 1921), range(2002, self.end_year)
        )
        self.assertNoHolidayName(name_2002, range(self.start_year, 2002))

    def test_feast_of_saint_simon_the_zealot(self):
        name = "Fiesta de San Simón Apóstol"
        self.assertHolidayName(name, (f"{year}-10-28" for year in range(self.start_year, 1918)))
        self.assertNoHolidayName(name, range(1918, self.end_year))

    def test_christmas_eve(self):
        self.assertHolidayName("Nochebuena", (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Día de Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_new_years_eve(self):
        self.assertHolidayName(
            "Fiesta de Fin de Año", (f"{year}-12-31" for year in self.full_range)
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2021-01-01", "Año Nuevo"),
            ("2021-02-15", "Lunes de Carnaval"),
            ("2021-02-16", "Martes de Carnaval"),
            ("2021-04-01", "Jueves Santo"),
            ("2021-04-02", "Viernes Santo"),
            ("2021-04-19", "Declaración de la Independencia"),
            ("2021-05-01", "Dia Mundial del Trabajador"),
            ("2021-06-24", "Batalla de Carabobo"),
            ("2021-07-05", "Día de la Independencia"),
            ("2021-07-24", "Natalicio de Simón Bolívar"),
            ("2021-10-12", "Día de la Resistencia Indígena"),
            ("2021-12-24", "Nochebuena"),
            ("2021-12-25", "Día de Navidad"),
            ("2021-12-31", "Fiesta de Fin de Año"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2021-01-01", "New Year's Day"),
            ("2021-02-15", "Carnival Monday"),
            ("2021-02-16", "Carnival Tuesday"),
            ("2021-04-01", "Maundy Thursday"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-19", "Declaration of Independence"),
            ("2021-05-01", "International Worker's Day"),
            ("2021-06-24", "Battle of Carabobo"),
            ("2021-07-05", "Independence Day"),
            ("2021-07-24", "Birthday of Simon Bolivar"),
            ("2021-10-12", "Day of Indigenous Resistance"),
            ("2021-12-24", "Christmas Eve"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2021-01-01", "Новий рік"),
            ("2021-02-15", "Карнавальний понеділок"),
            ("2021-02-16", "Карнавальний вівторок"),
            ("2021-04-01", "Великий четвер"),
            ("2021-04-02", "Страсна пʼятниця"),
            ("2021-04-19", "День проголошення незалежності"),
            ("2021-05-01", "Міжнародний день трудящих"),
            ("2021-06-24", "День битви при Карабобо"),
            ("2021-07-05", "День незалежності"),
            ("2021-07-24", "Річниця Сімона Болівара"),
            ("2021-10-12", "День спротиву корінних народів"),
            ("2021-12-24", "Святий вечір"),
            ("2021-12-25", "Різдво Христове"),
            ("2021-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import CATHOLIC, HEBREW, ISLAMIC, ORTHODOX, WORKDAY
from holidays.countries.montenegro import Montenegro
from tests.common import CommonCountryTests


class TestMontenegro(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2007, 2050)
        super().setUpClass(Montenegro, years=years, years_non_observed=years)
        cls.catholic_holidays = Montenegro(categories=CATHOLIC, years=years)
        cls.hebrew_holidays = Montenegro(categories=HEBREW, years=years)
        cls.islamic_holidays = Montenegro(categories=ISLAMIC, years=years)
        cls.orthodox_holidays = Montenegro(categories=ORTHODOX, years=years)

    def test_new_years_day(self):
        name = "Nova godina"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in range(2007, 2050)),
            (f"{year}-01-02" for year in range(2007, 2050)),
        )
        dt = (
            "2011-01-03",
            "2012-01-03",
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (neradni dan)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_labor_day(self):
        name = "Praznik rada"
        self.assertHolidayName(
            name,
            (f"{year}-05-01" for year in range(2007, 2050)),
            (f"{year}-05-02" for year in range(2007, 2050)),
        )
        dt = (
            "2010-05-03",
            "2011-05-03",
            "2016-05-03",
            "2021-05-03",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (neradni dan)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_independence_day(self):
        name = "Dan nezavisnosti"
        self.assertHolidayName(
            name,
            (f"{year}-05-21" for year in range(2007, 2050)),
            (f"{year}-05-22" for year in range(2007, 2050)),
        )
        dt = (
            "2011-05-23",
            "2016-05-23",
            "2017-05-23",
            "2022-05-23",
            "2023-05-23",
        )
        self.assertHolidayName(f"{name} (neradni dan)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_statehood_day(self):
        name = "Dan državnosti"
        self.assertHolidayName(
            name,
            (f"{year}-07-13" for year in range(2007, 2050)),
            (f"{year}-07-14" for year in range(2007, 2050)),
        )
        dt = (
            "2008-07-15",
            "2013-07-15",
            "2014-07-15",
            "2019-07-15",
            "2024-07-15",
        )
        self.assertHolidayName(f"{name} (neradni dan)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_njegos_day(self):
        name = "Njegošev dan"
        self.assertHolidayName(name, (f"{year}-11-13" for year in range(2022, 2050)))
        self.assertNoHolidayName(name, range(2007, 2022))
        dt = (
            "2022-11-14",
            "2033-11-14",
        )
        self.assertHolidayName(f"{name} (neradni dan)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_good_friday_catholic(self):
        name = "Veliki petak"
        self.assertNoHolidayName(name)
        dt = (
            "2015-04-03",
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, self.catholic_holidays, dt)
        self.assertHolidayName(name, self.catholic_holidays, range(2007, 2050))

    def test_easter_monday_catholic(self):
        name = "Uskrs"
        self.assertNoHolidayName(name)
        dt = (
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, self.catholic_holidays, dt)
        self.assertHolidayName(name, self.catholic_holidays, range(2007, 2050))

    def test_all_saints_day_catholic(self):
        name = "Svi Sveti"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.catholic_holidays, (f"{year}-11-01" for year in range(2007, 2050))
        )

    def test_christmas_eve_catholic(self):
        name = "Badnji dan"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.catholic_holidays, (f"{year}-12-24" for year in range(2007, 2050))
        )

    def test_christmas_day_catholic(self):
        name = "Božić"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name,
            self.catholic_holidays,
            (f"{year}-12-25" for year in range(2007, 2050)),
            (f"{year}-12-26" for year in range(2007, 2050)),
        )

    def test_pesach(self):
        name = "Pasha"
        self.assertNoHolidayName(name)
        dt = (
            "2015-04-05",
            "2015-04-04",
            "2016-04-23",
            "2016-04-24",
            "2017-04-11",
            "2024-04-24",
            "2017-04-12",
            "2018-03-31",
            "2018-04-01",
            "2019-04-20",
            "2019-04-21",
            "2020-04-09",
            "2020-04-10",
            "2021-03-28",
            "2021-03-29",
            "2022-04-16",
            "2022-04-17",
            "2023-04-06",
            "2023-04-07",
            "2024-04-23",
        )
        self.assertHolidayName(name, self.hebrew_holidays, dt)
        self.assertHolidayName(name, self.hebrew_holidays, range(2007, 2050))

    def test_yom_kippur(self):
        name = "Jom Kipur"
        self.assertNoHolidayName(name)
        dt = (
            "2015-09-23",
            "2015-09-24",
            "2016-10-12",
            "2016-10-13",
            "2017-09-30",
            "2017-10-01",
            "2018-09-19",
            "2018-09-20",
            "2019-10-09",
            "2019-10-10",
            "2020-09-28",
            "2020-09-29",
            "2021-09-16",
            "2021-09-17",
            "2022-10-05",
            "2022-10-06",
            "2023-09-25",
            "2023-09-26",
            "2024-10-12",
            "2024-10-13",
        )
        self.assertHolidayName(name, self.hebrew_holidays, dt)
        self.assertHolidayName(name, self.hebrew_holidays, range(2007, 2050))

    def test_eid_al_fitr(self):
        name = "Ramazanski bajram"
        self.assertNoHolidayName(name)
        dt = (
            "2015-07-18",
            "2015-07-19",
            "2015-07-20",
            "2016-07-07",
            "2016-07-08",
            "2016-07-09",
            "2017-06-26",
            "2017-06-27",
            "2017-06-28",
            "2018-06-15",
            "2018-06-16",
            "2018-06-17",
            "2019-06-04",
            "2019-06-05",
            "2019-06-06",
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
        )
        self.assertHolidayName(name, self.islamic_holidays, dt)

    def test_eid_al_adha(self):
        name = "Kurbanski bajram"
        self.assertNoHolidayName(name)
        dt = (
            "2015-09-24",
            "2015-09-25",
            "2015-09-26",
            "2016-09-13",
            "2016-09-14",
            "2016-09-15",
            "2017-09-02",
            "2017-09-03",
            "2017-09-04",
            "2018-08-22",
            "2018-08-23",
            "2018-08-24",
            "2019-08-11",
            "2019-08-12",
            "2019-08-13",
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
        )
        self.assertHolidayName(name, self.islamic_holidays, dt)

    def test_good_friday_orthodox(self):
        name = "Veliki petak"
        self.assertNoHolidayName(name)
        dt = (
            "2015-04-10",
            "2016-04-29",
            "2017-04-14",
            "2018-04-06",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
        )
        self.assertHolidayName(name, self.orthodox_holidays, dt)
        self.assertHolidayName(name, self.orthodox_holidays, range(2007, 2050))

    def test_easter_monday_orthodox(self):
        name = "Uskrs"
        self.assertNoHolidayName(name)
        dt = (
            "2015-04-13",
            "2016-05-02",
            "2017-04-17",
            "2018-04-09",
            "2019-04-29",
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
        )
        self.assertHolidayName(name, self.orthodox_holidays, dt)
        self.assertHolidayName(name, self.orthodox_holidays, range(2007, 2050))

    def test_christmas_eve_orthodox(self):
        name = "Badnji dan"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.orthodox_holidays, (f"{year}-01-06" for year in range(2007, 2050))
        )

    def test_christmas_day_orthodox(self):
        name = "Božić"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name,
            self.orthodox_holidays,
            (f"{year}-01-07" for year in range(2007, 2050)),
            (f"{year}-01-08" for year in range(2007, 2050)),
        )

    def test_ecological_state_day(self):
        name = "Dan Ekološke države"
        self.assertNoHolidayName(name)
        workday_holidays = Montenegro(categories=WORKDAY, years=range(2007, 2050))
        self.assertHolidayName(
            name, workday_holidays, (f"{year}-09-20" for year in range(2022, 2050))
        )
        self.assertNoHolidayName(name, workday_holidays, range(2007, 2022))

    def test_2022(self):
        self.assertHolidays(
            Montenegro(categories=self.holidays.supported_categories, years=2022),
            ("2022-01-01", "Nova godina"),
            ("2022-01-02", "Nova godina"),
            ("2022-01-03", "Nova godina (neradni dan)"),
            ("2022-01-06", "Badnji dan"),
            ("2022-01-07", "Božić"),
            ("2022-01-08", "Božić"),
            ("2022-04-15", "Veliki petak"),
            ("2022-04-16", "Pasha"),
            ("2022-04-17", "Pasha"),
            ("2022-04-18", "Uskrs"),
            ("2022-04-22", "Veliki petak"),
            ("2022-04-25", "Uskrs"),
            ("2022-05-01", "Praznik rada"),
            ("2022-05-02", "Praznik rada; Ramazanski bajram"),
            ("2022-05-03", "Praznik rada (neradni dan); Ramazanski bajram"),
            ("2022-05-04", "Ramazanski bajram"),
            ("2022-05-21", "Dan nezavisnosti"),
            ("2022-05-22", "Dan nezavisnosti"),
            ("2022-05-23", "Dan nezavisnosti (neradni dan)"),
            ("2022-07-09", "Kurbanski bajram"),
            ("2022-07-10", "Kurbanski bajram"),
            ("2022-07-11", "Kurbanski bajram"),
            ("2022-07-13", "Dan državnosti"),
            ("2022-07-14", "Dan državnosti"),
            ("2022-09-20", "Dan Ekološke države"),
            ("2022-10-05", "Jom Kipur"),
            ("2022-10-06", "Jom Kipur"),
            ("2022-11-01", "Svi Sveti"),
            ("2022-11-13", "Njegošev dan"),
            ("2022-11-14", "Njegošev dan (neradni dan)"),
            ("2022-12-24", "Badnji dan"),
            ("2022-12-25", "Božić"),
            ("2022-12-26", "Božić"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Nova godina"),
            ("2023-01-02", "Nova godina"),
            ("2023-01-03", "Nova godina (neradni dan)"),
            ("2023-01-06", "Badnji dan"),
            ("2023-01-07", "Božić"),
            ("2023-01-08", "Božić"),
            ("2023-04-06", "Pasha"),
            ("2023-04-07", "Pasha; Veliki petak"),
            ("2023-04-10", "Uskrs"),
            ("2023-04-14", "Veliki petak"),
            ("2023-04-17", "Uskrs"),
            ("2023-04-21", "Ramazanski bajram"),
            ("2023-04-22", "Ramazanski bajram"),
            ("2023-04-23", "Ramazanski bajram"),
            ("2023-05-01", "Praznik rada"),
            ("2023-05-02", "Praznik rada"),
            ("2023-05-21", "Dan nezavisnosti"),
            ("2023-05-22", "Dan nezavisnosti"),
            ("2023-05-23", "Dan nezavisnosti (neradni dan)"),
            ("2023-06-28", "Kurbanski bajram"),
            ("2023-06-29", "Kurbanski bajram"),
            ("2023-06-30", "Kurbanski bajram"),
            ("2023-07-13", "Dan državnosti"),
            ("2023-07-14", "Dan državnosti"),
            ("2023-09-20", "Dan Ekološke države"),
            ("2023-09-25", "Jom Kipur"),
            ("2023-09-26", "Jom Kipur"),
            ("2023-11-01", "Svi Sveti"),
            ("2023-11-13", "Njegošev dan"),
            ("2023-12-24", "Badnji dan"),
            ("2023-12-25", "Božić"),
            ("2023-12-26", "Božić"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day"),
            ("2023-01-03", "New Year's Day (observed)"),
            ("2023-01-06", "Christmas Eve"),
            ("2023-01-07", "Christmas"),
            ("2023-01-08", "Christmas"),
            ("2023-04-06", "Pesach"),
            ("2023-04-07", "Good Friday; Pesach"),
            ("2023-04-10", "Easter"),
            ("2023-04-14", "Good Friday"),
            ("2023-04-17", "Easter"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-02", "Labor Day"),
            ("2023-05-21", "Independence Day"),
            ("2023-05-22", "Independence Day"),
            ("2023-05-23", "Independence Day (observed)"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Eid al-Adha"),
            ("2023-07-13", "Statehood Day"),
            ("2023-07-14", "Statehood Day"),
            ("2023-09-20", "Ecological State Day"),
            ("2023-09-25", "Yom Kippur"),
            ("2023-09-26", "Yom Kippur"),
            ("2023-11-01", "All Saints' Day"),
            ("2023-11-13", "Njegos Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas"),
            ("2023-12-26", "Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-02", "Новий рік"),
            ("2023-01-03", "Новий рік (вихідний)"),
            ("2023-01-06", "Святий вечір"),
            ("2023-01-07", "Різдво Христове"),
            ("2023-01-08", "Різдво Христове"),
            ("2023-04-06", "Песах"),
            ("2023-04-07", "Песах; Страсна пʼятниця"),
            ("2023-04-10", "Великдень"),
            ("2023-04-14", "Страсна пʼятниця"),
            ("2023-04-17", "Великдень"),
            ("2023-04-21", "Рамазан-байрам"),
            ("2023-04-22", "Рамазан-байрам"),
            ("2023-04-23", "Рамазан-байрам"),
            ("2023-05-01", "День праці"),
            ("2023-05-02", "День праці"),
            ("2023-05-21", "День незалежності"),
            ("2023-05-22", "День незалежності"),
            ("2023-05-23", "День незалежності (вихідний)"),
            ("2023-06-28", "Курбан-байрам"),
            ("2023-06-29", "Курбан-байрам"),
            ("2023-06-30", "Курбан-байрам"),
            ("2023-07-13", "День державності"),
            ("2023-07-14", "День державності"),
            ("2023-09-20", "День екологічної держави"),
            ("2023-09-25", "Йом Кіпур"),
            ("2023-09-26", "Йом Кіпур"),
            ("2023-11-01", "День усіх святих"),
            ("2023-11-13", "День Нєґоша"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-26", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.qatar import Qatar
from tests.common import CommonCountryTests


class TestQatar(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Qatar)

    def test_special_holidays(self):
        self.assertHoliday(
            "2025-01-02",
        )

    def test_sports_day(self):
        name = "اليوم الوطني للرياضة"
        self.assertHolidayName(
            name,
            "2012-02-14",
            "2013-02-12",
            "2014-02-11",
            "2015-02-10",
            "2016-02-09",
            "2017-02-14",
            "2018-02-13",
            "2019-02-12",
            "2020-02-11",
            "2021-02-09",
            "2022-02-08",
            "2023-02-14",
            "2024-02-13",
            "2025-02-11",
            "2026-02-10",
            "2027-02-09",
            "2028-02-08",
            "2029-02-13",
            "2030-02-12",
        )
        self.assertHolidayName(name, range(2012, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_qatar_national_day(self):
        name = "اليوم الوطني لقطر"
        self.assertHolidayName(name, (f"{year}-12-18" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_new_years_day(self):
        name = "رأس السنة الميلادية"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-01-01" for year in self.full_range))

    def test_march_bank_holiday(self):
        name = "عطلة البنك"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-03-01",
            "2021-03-07",
            "2022-03-06",
            "2023-03-05",
            "2024-03-03",
            "2025-03-02",
        )
        self.assertBankHolidayName(name, range(2010, self.end_year))
        self.assertNoBankHolidayName(name, range(self.start_year, 2010))

    def test_weekend(self):
        for dt in (
            "2003-07-24",  # THU.
            "2003-07-25",  # FRI.
            "2003-07-31",  # THU.
            "2003-08-01",  # FRI.
            "2003-08-02",  # SAT.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "2003-07-26",  # SAT.
            "2003-07-27",  # SUN.
            "2003-08-03",  # SUN.
            "2003-08-07",  # THU.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_2011(self):
        self.assertHolidaysInYear(
            2011,
            ("2011-08-31", "عيد الفطر"),
            ("2011-09-01", "عيد الفطر"),
            ("2011-09-02", "عيد الفطر"),
            ("2011-11-06", "عيد الأضحى"),
            ("2011-11-07", "عيد الأضحى"),
            ("2011-11-08", "عيد الأضحى"),
            ("2011-12-18", "اليوم الوطني لقطر"),
        )

    def test_bank_2024(self):
        self.assertBankHolidaysInYear(
            2024,
            ("2024-01-01", "رأس السنة الميلادية"),
            ("2024-03-03", "عطلة البنك"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2012-01-01", "رأس السنة الميلادية"),
            ("2012-02-14", "اليوم الوطني للرياضة"),
            ("2012-03-04", "عطلة البنك"),
            ("2012-08-19", "عيد الفطر"),
            ("2012-08-20", "عيد الفطر"),
            ("2012-08-21", "عيد الفطر"),
            ("2012-10-26", "عيد الأضحى"),
            ("2012-10-27", "عيد الأضحى"),
            ("2012-10-28", "عيد الأضحى"),
            ("2012-12-18", "اليوم الوطني لقطر"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2012-01-01", "New Year's Day"),
            ("2012-02-14", "National Sports Day"),
            ("2012-03-04", "March Bank Holiday"),
            ("2012-08-19", "Eid al-Fitr"),
            ("2012-08-20", "Eid al-Fitr"),
            ("2012-08-21", "Eid al-Fitr"),
            ("2012-10-26", "Eid al-Adha"),
            ("2012-10-27", "Eid al-Adha"),
            ("2012-10-28", "Eid al-Adha"),
            ("2012-12-18", "Qatar National Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

import holidays


class TestHolidaysImports(TestCase):
    def assertImport(self, name):  # noqa: N802
        self.assertTrue(hasattr(holidays, name), f"Import error: `from holidays import {name}`")

    def test_constants(self):
        for name in (
            "MON",
            "TUE",
            "WED",
            "THU",
            "FRI",
            "SAT",
            "SUN",
            "WEEKEND",
        ):
            self.assertImport(name)

    def test_holidays_base(self):
        for name in ("DateLike", "HolidayBase", "HolidaySum"):
            self.assertImport(name)

    def test_utils(self):
        for name in (
            "country_holidays",
            "CountryHoliday",
            "financial_holidays",
            "list_localized_countries",
            "list_localized_financial",
            "list_supported_countries",
            "list_supported_financial",
        ):
            self.assertImport(name)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bangladesh import Bangladesh
from tests.common import CommonCountryTests


class TestBangladesh(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bangladesh)

    def test_martyrs_day_and_international_mother_language_day(self):
        self.assertHolidayName(
            "শহীদ দিবস ও আন্তর্জাতিক মাতৃভাষা দিবস", (f"{year}-02-21" for year in self.full_range)
        )

    def test_sheikh_mujibur_rahmans_birthday(self):
        name = "জাতির পিতা বঙ্গবন্ধু শেখ মুজিবুর রহমান এর জন্মদিবস"
        self.assertHolidayName(name, (f"{year}-03-17" for year in range(self.start_year, 2025)))
        self.assertNoHolidayName(name, range(2025, self.end_year))

    def test_independence_day(self):
        self.assertHolidayName("স্বাধীনতা দিবস", (f"{year}-03-26" for year in self.full_range))

    def test_bengali_new_years_day(self):
        self.assertHolidayName("পহেলা বৈশাখ", (f"{year}-04-14" for year in self.full_range))

    def test_may_day(self):
        self.assertHolidayName("মে দিবস", (f"{year}-05-01" for year in self.full_range))

    def test_july_mass_uprising_day(self):
        name = "জুলাই গণ-অভ্যুত্থান দিবস"
        self.assertHolidayName(name, (f"{year}-08-05" for year in range(2025, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2025))

    def test_national_mourning_day(self):
        name = "জাতীয় শোক দিবস"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(self.start_year, 2025)))
        self.assertNoHolidayName(name, range(2025, self.end_year))

    def test_victory_day(self):
        self.assertHolidayName("বিজয় দিবস", (f"{year}-12-16" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("বড়দিন", (f"{year}-12-25" for year in self.full_range))

    def test_ashura_day(self):
        name = "আশুরা"
        self.assertHolidayName(
            name,
            "2022-08-09",
            "2023-07-29",
            "2024-07-17",
            "2025-07-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "ঈদে মিলাদুন্নবী"
        self.assertHolidayName(
            name,
            "2022-10-09",
            "2023-09-28",
            "2024-09-16",
            "2025-09-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_mid_shaban(self):
        name = "শবে বরাত"
        self.assertHolidayName(
            name,
            "2022-03-19",
            "2023-03-08",
            "2024-02-26",
            "2025-02-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_laylat_al_qadr(self):
        name = "শবে কদর"
        self.assertHolidayName(
            name,
            "2022-04-29",
            "2023-04-19",
            "2024-04-07",
            "2025-03-28",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_jumuatul_wida(self):
        name = "জুমাতুল বিদা"
        self.assertHolidayName(
            name,
            "2022-04-29",
            "2023-04-21",
            "2024-04-05",
            "2025-03-28",
            "2026-03-20",
        )

    def test_eid_al_fitr(self):
        name = "ঈদুল ফিতর"
        self.assertHolidayName(
            name,
            "2023-04-22",
            "2023-04-23",
            "2023-04-24",
            "2022-05-03",
            "2022-05-04",
            "2022-05-05",
            "2024-04-11",
            "2024-04-12",
            "2024-04-10",
            "2025-03-31",
            "2025-04-01",
            "2025-04-02",
        )
        exception_years = {1968, 2000, 2033}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, (year for year in self.full_range if year not in exception_years)
        )

        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, exception_years)

    def test_eid_al_adha(self):
        name = "ঈদুল আজহা"
        self.assertHolidayName(
            name,
            "2022-07-11",
            "2022-07-12",
            "2022-07-10",
            "2023-06-29",
            "2023-06-30",
            "2023-07-01",
            "2024-06-17",
            "2024-06-18",
            "2024-06-19",
            "2025-06-07",
            "2025-06-08",
            "2025-06-09",
        )
        exception_years = {1974, 2006, 2007, 2039}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, (year for year in self.full_range if year not in exception_years)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 2006)

        self.assertIslamicNoEstimatedHolidayNameCount(name, 5, 2007)

        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, 1974, 2039)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-02-21", "শহীদ দিবস ও আন্তর্জাতিক মাতৃভাষা দিবস"),
            ("2024-02-26", "শবে বরাত"),
            ("2024-03-17", "জাতির পিতা বঙ্গবন্ধু শেখ মুজিবুর রহমান এর জন্মদিবস"),
            ("2024-03-26", "স্বাধীনতা দিবস"),
            ("2024-04-05", "জুমাতুল বিদা"),
            ("2024-04-07", "শবে কদর"),
            ("2024-04-10", "ঈদুল ফিতর"),
            ("2024-04-11", "ঈদুল ফিতর"),
            ("2024-04-12", "ঈদুল ফিতর"),
            ("2024-04-14", "পহেলা বৈশাখ"),
            ("2024-05-01", "মে দিবস"),
            ("2024-06-17", "ঈদুল আজহা"),
            ("2024-06-18", "ঈদুল আজহা"),
            ("2024-06-19", "ঈদুল আজহা"),
            ("2024-07-17", "আশুরা"),
            ("2024-08-15", "জাতীয় শোক দিবস"),
            ("2024-09-16", "ঈদে মিলাদুন্নবী"),
            ("2024-12-16", "বিজয় দিবস"),
            ("2024-12-25", "বড়দিন"),
        )

    def test_l10n_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2024-02-21", "يوم الشهداء واليوم الدولي للغة الأم"),
            ("2024-02-26", "ليلة النصف من شعبان"),
            ("2024-03-17", "عيد ميلاد الشيخ مجيب الرحمن"),
            ("2024-03-26", "عيد الاستقلال"),
            ("2024-04-05", "جمعة الوداع"),
            ("2024-04-07", "ليلة القدر"),
            ("2024-04-10", "عيد الفطر"),
            ("2024-04-11", "عيد الفطر"),
            ("2024-04-12", "عيد الفطر"),
            ("2024-04-14", "رأس السنة البنغالية"),
            ("2024-05-01", "الأول من مايو"),
            ("2024-06-17", "عيد الأضحى"),
            ("2024-06-18", "عيد الأضحى"),
            ("2024-06-19", "عيد الأضحى"),
            ("2024-07-17", "عاشوراء"),
            ("2024-08-15", "اليوم الوطني للحداد"),
            ("2024-09-16", "المولد النبوي الشريف"),
            ("2024-12-16", "عيد النصر"),
            ("2024-12-25", "عيد الميلاد"),
        )

    def test_l10n_en_bd(self):
        self.assertLocalizedHolidays(
            "en_BD",
            ("2024-02-21", "Shaheed Day and International Mother Language Day"),
            ("2024-02-26", "Shab-e-Barat"),
            ("2024-03-17", "Father of the Nation Bangabandhu Sheikh Mujibur Rahman's Birthday"),
            ("2024-03-26", "Independence Day"),
            ("2024-04-05", "Jumu'atul-Wida"),
            ("2024-04-07", "Shab-e-Qadr"),
            ("2024-04-10", "Eid-ul-Fitr"),
            ("2024-04-11", "Eid-ul-Fitr"),
            ("2024-04-12", "Eid-ul-Fitr"),
            ("2024-04-14", "Pohela Boishakh"),
            ("2024-05-01", "May Day"),
            ("2024-06-17", "Eid-ul-Adha"),
            ("2024-06-18", "Eid-ul-Adha"),
            ("2024-06-19", "Eid-ul-Adha"),
            ("2024-07-17", "Ashura"),
            ("2024-08-15", "National Mourning Day"),
            ("2024-09-16", "Eid-e-Miladunnabi"),
            ("2024-12-16", "Victory Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-02-21", "Martyrs' Day and International Mother Language Day"),
            ("2024-02-26", "Mid-Sha'ban"),
            ("2024-03-17", "Sheikh Mujibur Rahman's Birthday"),
            ("2024-03-26", "Independence Day"),
            ("2024-04-05", "Jumu'atul-Wida"),
            ("2024-04-07", "Laylat al-Qadr"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-11", "Eid al-Fitr"),
            ("2024-04-12", "Eid al-Fitr"),
            ("2024-04-14", "Bengali New Year's Day"),
            ("2024-05-01", "May Day"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-06-18", "Eid al-Adha"),
            ("2024-06-19", "Eid al-Adha"),
            ("2024-07-17", "Ashura"),
            ("2024-08-15", "National Mourning Day"),
            ("2024-09-16", "Prophet's Birthday"),
            ("2024-12-16", "Victory Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.gambia import Gambia
from tests.common import CommonCountryTests


class TestGambia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Gambia)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dt = (
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-02-18" for year in self.full_range))
        obs_dt = (
            "2023-02-20",
            "2024-02-19",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dt = (
            "2021-05-03",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)

    def test_africa_liberation_day(self):
        name = "Africa Liberation Day"
        self.assertHolidayName(name, (f"{year}-05-25" for year in self.full_range))
        obs_dt = (
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_july_22_revolution_day(self):
        name = "July 22 Revolution Day"
        self.assertHolidayName(name, (f"{year}-07-22" for year in self.full_range))
        obs_dt = ("2023-07-24",)
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_assumption_day(self):
        name = "Feast of the Assumption"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dt = (
            "2021-08-16",
            "2026-08-17",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dt = (
            "2021-12-27",
            "2026-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_ashura(self):
        name = "Yawmul Ashura"
        self.assertHolidayName(
            name,
            "2021-08-19",
            "2022-08-08",
            "2023-07-28",
            "2024-07-16",
            "2025-07-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dt = (
            "2025-07-07",
            "2028-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_prophets_birthday(self):
        name = "Mawlid Nabi"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-28",
            "2024-09-16",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dt = (
            "2022-10-10",
            "2027-08-16",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_laylat_al_qadr(self):
        name = "Lialat-Ul-Qadr"
        self.assertHolidayName(
            name,
            "2021-05-09",
            "2022-04-29",
            "2023-04-18",
            "2024-04-06",
            "2025-03-27",
        )
        obs_dt = (
            "2021-05-10",
            "2024-04-08",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_eid_al_fitr(self):
        name = "Koriteh"
        self.assertHolidayName(
            name,
            "2021-05-13",
            "2021-05-14",
            "2022-05-02",
            "2022-05-03",
            "2023-04-21",
            "2023-04-22",
            "2024-04-10",
            "2024-04-11",
            "2025-03-30",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dt = (
            "2023-04-24",
            "2026-03-23",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_eid_al_adha(self):
        name = "Tobaski"
        self.assertHolidayName(
            name,
            "2021-07-20",
            "2021-07-21",
            "2022-07-09",
            "2022-07-10",
            "2023-06-28",
            "2023-06-29",
            "2024-06-16",
            "2024-06-17",
            "2025-06-06",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dt = (
            "2022-07-11",
            "2025-06-09",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-18", "Independence Day"),
            ("2024-02-19", "Independence Day (observed)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-06", "Lialat-Ul-Qadr"),
            ("2024-04-08", "Lialat-Ul-Qadr (observed)"),
            ("2024-04-10", "Koriteh"),
            ("2024-04-11", "Koriteh"),
            ("2024-05-01", "Labour Day"),
            ("2024-05-25", "Africa Liberation Day"),
            ("2024-05-27", "Africa Liberation Day (observed)"),
            ("2024-06-16", "Tobaski"),
            ("2024-06-17", "Tobaski"),
            ("2024-07-16", "Yawmul Ashura"),
            ("2024-07-22", "July 22 Revolution Day"),
            ("2024-08-15", "Feast of the Assumption"),
            ("2024-09-16", "Mawlid Nabi"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-02-18", "Independence Day"),
            ("2025-03-27", "Lialat-Ul-Qadr"),
            ("2025-03-30", "Koriteh"),
            ("2025-03-31", "Koriteh"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labour Day"),
            ("2025-05-25", "Africa Liberation Day"),
            ("2025-05-26", "Africa Liberation Day (observed)"),
            ("2025-06-06", "Tobaski"),
            ("2025-06-07", "Tobaski"),
            ("2025-06-09", "Tobaski (observed)"),
            ("2025-07-05", "Yawmul Ashura"),
            ("2025-07-07", "Yawmul Ashura (observed)"),
            ("2025-07-22", "July 22 Revolution Day"),
            ("2025-08-15", "Feast of the Assumption"),
            ("2025-09-04", "Mawlid Nabi (estimated)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-02-18", "Independence Day"),
            ("2025-03-27", "Laylat al-Qadr"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-25", "Africa Liberation Day"),
            ("2025-05-26", "Africa Liberation Day (observed)"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-06-09", "Eid al-Adha (observed)"),
            ("2025-07-05", "Ashura"),
            ("2025-07-07", "Ashura (observed)"),
            ("2025-07-22", "July 22 Revolution Day"),
            ("2025-08-15", "Assumption Day"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import json
import shutil
import sys
from concurrent.futures import ProcessPoolExecutor
from pathlib import Path

sys.path.insert(0, str(Path.cwd()))  # Make holidays visible.

from holidays import (
    country_holidays,
    financial_holidays,
    list_supported_countries,
    list_supported_financial,
)
from holidays.ical import ICalExporter

# Configuration Defaults
DEFAULT_YEAR_START = 2015
DEFAULT_YEAR_END = 2035

OUTPUT_DIR = Path("site/downloads/ics")

LANGUAGES = {
    # --- Base Languages ---
    "am": "Amharic",
    "ar": "Arabic",
    "az": "Azerbaijani",
    "be": "Belarusian",
    "bg": "Bulgarian",
    "bn": "Bengali",
    "bs": "Bosnian",
    "ca": "Catalan",
    "cnr": "Montenegrin",
    "cs": "Czech",
    "da": "Danish",
    "de": "German",
    "dz": "Dzongkha",
    "el": "Greek",
    "es": "Spanish",
    "et": "Estonian",
    "fi": "Finnish",
    "fil": "Filipino",
    "fo": "Faroese",
    "fr": "French",
    "fy": "Western Frisian",
    "gu": "Gujarati",
    "he": "Hebrew",
    "hi": "Hindi",
    "hr": "Croatian",
    "ht": "Haitian Creole",
    "hu": "Hungarian",
    "hy": "Armenian",
    "id": "Indonesian",
    "is": "Icelandic",
    "it": "Italian",
    "ja": "Japanese",
    "ka": "Georgian",
    "kk": "Kazakh",
    "kl": "Kalaallisut (Greenlandic)",
    "km": "Khmer",
    "kn": "Kannada",
    "ko": "Korean",
    "ky": "Kirghiz",
    "lb": "Luxembourgish",
    "lo": "Lao",
    "lt": "Lithuanian",
    "lv": "Latvian",
    "mg": "Malagasy",
    "mk": "Macedonian",
    "ml": "Malayalam",
    "mn": "Mongolian",
    "mr": "Marathi",
    "ms": "Malay",
    "mt": "Maltese",
    "my": "Burmese",
    "ne": "Nepali",
    "nl": "Dutch",
    "no": "Norwegian",
    "pa": "Panjabi",
    "pl": "Polish",
    "ro": "Romanian",
    "ru": "Russian",
    "rw": "Kinyarwanda",
    "sk": "Slovak",
    "sl": "Slovenian",
    "sq": "Albanian",
    "sr": "Serbian",
    "sv": "Swedish",
    "sw": "Swahili",
    "ta": "Tamil",
    "te": "Telugu",
    "tet": "Tetum",
    "tg": "Tajik",
    "th": "Thai",
    "tk": "Turkmen",
    "tkl": "Tokelauan",
    "to": "Tongan",
    "tr": "Turkish",
    "tvl": "Tuvaluan",
    "uk": "Ukrainian",
    "uz": "Uzbek",
    "vi": "Vietnamese",
    # --- Arabic Locales ---
    "ar_EG": "Arabic (Egypt)",
    "ar_QA": "Arabic (Qatar)",
    "ar_SD": "Arabic (Sudan)",
    # --- Cocos Islands ---
    "coa_CC": "Cocos Islands Malay",
    # --- English Locales ---
    "en_AI": "English (Anguilla)",
    "en_AU": "English (Australia)",
    "en_BF": "English (Burkina Faso)",
    "en_BD": "English (Bangladesh)",
    "en_BM": "English (Bermuda)",
    "en_BQ": "English (Bonaire, Sint Eustatius and Saba)",
    "en_CA": "English (Canada)",
    "en_CC": "English (Cocos Islands)",
    "en_CI": "English (Ivory Coast)",
    "en_CK": "English (Cook Islands)",
    "en_CX": "English (Christmas Island)",
    "en_CY": "English (Cyprus)",
    "en_ET": "English (Ethiopia)",
    "en_FM": "English (Micronesia)",
    "en_GB": "English (United Kingdom)",
    "en_GD": "English (Grenada)",
    "en_GM": "English (Gambia)",
    "en_GS": "English (South Georgia & South Sandwich Islands)",
    "en_GY": "English (Guyana)",
    "en_HK": "English (Hong Kong)",
    "en_IN": "English (India)",
    "en_KE": "English (Kenya)",
    "en_LC": "English (Saint Lucia)",
    "en_MO": "English (Macau)",
    "en_MS": "English (Montserrat)",
    "en_MU": "English (Mauritius)",
    "en_NA": "English (Namibia)",
    "en_NF": "English (Norfolk Island)",
    "en_NG": "English (Nigeria)",
    "en_NR": "English (Nauru)",
    "en_NU": "English (Niue)",
    "en_PH": "English (Philippines)",
    "en_PK": "English (Pakistan)",
    "en_SC": "English (Seychelles)",
    "en_SG": "English (Singapore)",
    "en_SL": "English (Sierra Leone)",
    "en_TC": "English (Turks and Caicos Islands)",
    "en_TK": "English (Tokelau)",
    "en_TL": "English (Timor-Leste)",
    "en_TT": "English (Trinidad and Tobago)",
    "en_US": "English (United States)",
    "en_VC": "English (Saint Vincent and the Grenadines)",
    "en_VG": "English (British Virgin Islands)",
    # --- Persian / Pashto Locales ---
    "fa_AF": "Persian (Afghanistan)",
    "fa_IR": "Persian (Iran)",
    "ps_AF": "Pashto (Afghanistan)",
    # --- French Locales ---
    "fr_BI": "French (Burundi)",
    "fr_BJ": "French (Benin)",
    "fr_HT": "French (Haiti)",
    "fr_NE": "French (Niger)",
    "fr_SN": "French (Senegal)",
    # --- Italian Locales ---
    "it_IT": "Italian (Italy)",
    # --- Korean Locales ---
    "ko_KP": "Korean (North Korea)",
    # --- Malay Locales ---
    "ms_MY": "Malay (Malaysia)",
    # --- Papiamento Locales ---
    "pap_AW": "Papiamento (Aruba)",
    "pap_BQ": "Papiamento (Bonaire, Sint Eustatius and Saba)",
    "pap_CW": "Papiamento (Curaçao)",
    # --- Portuguese Locales ---
    "pt_AO": "Portuguese (Angola)",
    "pt_BR": "Portuguese (Brazil)",
    "pt_CV": "Portuguese (Cabo Verde)",
    "pt_GW": "Portuguese (Guinea-Bissau)",
    "pt_MO": "Portuguese (Macau)",
    "pt_MZ": "Portuguese (Mozambique)",
    "pt_PT": "Portuguese (Portugal)",
    "pt_ST": "Portuguese (Sao Tome and Principe)",
    "pt_TL": "Portuguese (Timor Leste)",
    # --- Russian Locales ---
    "ru_KG": "Russian (Kyrgyzstan)",
    # --- Sinhala / Tamil Locales ---
    "si_LK": "Sinhala (Sri Lanka)",
    "ta_LK": "Tamil (Sri Lanka)",
    # --- Swedish Locales ---
    "sv_FI": "Swedish (Finland)",
    # --- Urdu Locales ---
    "ur_PK": "Urdu (Pakistan)",
    # --- Chinese Locales ---
    "zh_CN": "Chinese (Simplified)",
    "zh_HK": "Chinese (Hong Kong)",
    "zh_MO": "Chinese (Macau)",
    "zh_TW": "Chinese (Traditional)",
}


def write_assets(h_obj, filename_base, year_dir):
    """Write the ICS and JSON files to the disk."""
    if not h_obj:
        return
    try:
        exporter = ICalExporter(h_obj)
        ics_path = year_dir / f"{filename_base}.ics"
        exporter.save_ics(str(ics_path))

        # Inline JSON generation
        json_data = json.dumps(
            [{"date": dt.isoformat(), "name": name} for dt, name in sorted(h_obj.items())],
            ensure_ascii=False,
        )
        (year_dir / f"{filename_base}.json").write_text(json_data, encoding="utf-8", newline="\n")
    except Exception as e:
        print(f"Error writing {filename_base}: {e}")


def process_entity(args):
    """Worker that generates assets for a single Country/Market."""
    code, is_country = args

    holidays_func = country_holidays if is_country else financial_holidays
    instance = holidays_func(code)

    # Prepare metadata.
    docstring = instance.__class__.__base__.__doc__
    name = docstring.split("\n")[0].split("holidays")[0].strip()
    languages = instance.supported_languages or ["en_US"]
    categories = instance.supported_categories
    default_lang = getattr(instance, "default_language", None) or "en_US"

    subdivision_aliases = instance.get_subdivision_aliases()
    subdivisions_map = {
        subdiv: subdivision_aliases[subdiv][0] if subdivision_aliases[subdiv] else subdiv
        for subdiv in instance.subdivisions
    }

    # Generate a clean dictionary of full language names
    languages_map = {lang: LANGUAGES.get(lang, lang) for lang in sorted(languages)}

    manifest_entry = {
        "name": name,
        "subdivisions": subdivisions_map,
        "languages": languages_map,
        "categories": sorted(categories),
        "default_language": default_lang,
    }

    print(f"Processing {code} ({name})...")

    entity_type = "countries" if is_country else "financial"

    # Generate files
    for year in range(DEFAULT_YEAR_START, DEFAULT_YEAR_END + 1):
        year_dir = OUTPUT_DIR / entity_type / code / str(year)
        year_dir.mkdir(parents=True, exist_ok=True)

        for subdiv in (None, *instance.subdivisions):
            for lang in languages:
                for cat in categories:
                    filename = f"{subdiv or 'ALL'}_{lang}_{cat}"
                    try:
                        h_obj = holidays_func(
                            code, subdiv=subdiv, years=year, language=lang, categories=cat
                        )
                        write_assets(h_obj, filename, year_dir)
                    except Exception as e:
                        print(f"Failed {filename} for {code} {year}: {e}")

    return entity_type, code, manifest_entry


def main():
    print(f"--- Generating to {OUTPUT_DIR} ---")

    # Inline output dir cleaning
    if OUTPUT_DIR.exists():
        shutil.rmtree(OUTPUT_DIR)
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)

    work_items = [(c, True) for c in list_supported_countries(include_aliases=False)]
    work_items += [(m, False) for m in list_supported_financial(include_aliases=False)]

    print(f"Found {len(work_items)} entities to process.")
    manifest = {"countries": {}, "financial": {}}

    with ProcessPoolExecutor() as executor:
        for result in executor.map(process_entity, work_items):
            if result:
                etype, code, meta = result
                manifest[etype][code] = meta

    manifest_path = OUTPUT_DIR / "index.json"
    print(f"Writing manifest to {manifest_path}...")
    manifest_path.write_text(
        json.dumps(manifest, ensure_ascii=False, indent=2), encoding="utf-8", newline="\n"
    )

    print("Done.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate Gregorian dates for holidays based on the Hijri lunar calendar.

Run with:

    python -m scripts.calendar.islamic_generator

Alternatively, run with uv:

    uv run -m scripts.calendar.islamic_generator

This generates the file `holidays/calendars/islamic_dates.py`,
whose data can then be copied to `holidays/calendars/islamic.py`.
"""

from __future__ import annotations

from collections import defaultdict
from datetime import timedelta
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from datetime import date

from hijridate.convert import Hijri
from hijridate.ummalqura import HIJRI_RANGE

from .generator import CalendarGenerator

# holiday_name: (month, day)
#     Fixed Hijri date, e.g. "EID_AL_FITR": (10, 1).
#     Negative day values count from the end of the month,
#     e.g. "ALI_AL_RIDA_DEATH": (2, -1) for the last day of Safar.
# holiday_name: (month, nth, weekday)
#     n-th weekday of Hijri month, e.g. "JUMUATUL_WIDA": (9, -1, 4)
#     for the last Friday of Ramadan.
ISLAMIC_HOLIDAYS = {
    "HIJRI_NEW_YEAR": (1, 1),  # Islamic New Year.
    "TASUA": (1, 9),  # Tasua.
    "ASHURA": (1, 10),  # Ashura.
    "HARI_HOL_JOHOR": (2, 6),  # Hari Hol of Johor.
    "GRAND_MAGAL_OF_TOUBA": (2, 18),  # Grand Magal of Touba.
    "ARBAEEN": (2, 20),  # Arbaeen.
    "PROPHET_DEATH": (2, 28),  # Demise of Prophet Muhammad and Martyrdom of Imam Hassan.
    "ALI_AL_RIDA_DEATH": (2, -1),  # Martyrdom of Imam Reza (last day of Safar).
    "QUAMEE_DHUVAS": (3, 1),  # National Day in Maldives.
    "HASAN_AL_ASKARI_DEATH": (3, 8),  # Martyrdom of Imam Hasan al-Askari.
    "MAWLID": (3, 12),  # Birthday of Prophet.
    "SADIQ_BIRTHDAY": (3, 17),  # Birthday of Prophet Muhammad and Imam Sadeq.
    "MALDIVES_EMBRACED_ISLAM_DAY": (4, 1),  # The Day Maldives Embraced Islam.
    "FATIMA_DEATH": (6, 3),  # Martyrdom of Fatima.
    "ALI_BIRTHDAY": (7, 13),  # Birthday of Imam Ali.
    "ISRA_AND_MIRAJ": (7, 27),  # Isra and Mi'raj.
    "IMAM_MAHDI_BIRTHDAY": (8, 15),  # Birthday of Imam Mahdi.
    "RAMADAN_BEGINNING": (9, 1),  # Beginning of Ramadan.
    "NUZUL_AL_QURAN": (9, 17),  # Nuzul Al-Quran Day.
    "ALI_DEATH": (9, 21),  # Martyrdom of Imam Ali.
    "LAYLAT_AL_QADR": (9, 27),  # Laylat Al-Qadr.
    "JUMUATUL_WIDA": (9, -1, 4),  # Jumuatul-Wida (last Friday of Ramadan).
    "EID_AL_FITR": (10, 1),  # Eid al-Fitr.
    "SADIQ_DEATH": (10, 25),  # Martyrdom of Imam Sadeq.
    "EID_AL_ADHA": (12, 10),  # Eid al-Adha.
    "EID_AL_GHADIR": (12, 18),  # Eid al-Ghadir.
}


def _get_nth_weekday_from(n: int, weekday: int, from_dt: date) -> date:
    """Return the n-th occurrence of weekday relative to given date.

    Positive `n` counts forward, negative `n` counts backward,
    including `from_dt` itself.
    """
    return from_dt + timedelta(
        (n - 1) * 7 + (weekday - from_dt.weekday()) % 7
        if n > 0
        else (n + 1) * 7 - (from_dt.weekday() - weekday) % 7
    )


def generate_data() -> None:
    h_years = range(HIJRI_RANGE[0][0], HIJRI_RANGE[1][0] + 1)

    dates: dict[str, dict[int, list[date]]] = defaultdict(lambda: defaultdict(list))
    for name, (holiday_month, *items) in ISLAMIC_HOLIDAYS.items():
        for year in h_years:
            first_day = Hijri(year, holiday_month, 1)
            g_date = first_day.to_gregorian()

            match items:
                case (holiday_num, holiday_weekday):
                    if holiday_num < 0:
                        g_date += timedelta(days=first_day.month_length() - 1)
                    g_date = _get_nth_weekday_from(holiday_num, holiday_weekday, g_date)

                case (holiday_day,):
                    if holiday_day < 0:
                        g_date += timedelta(days=first_day.month_length() + 1)
                    g_date += timedelta(days=holiday_day - 1)

            dates[name][g_date.year].append(g_date)

    cal_gen = CalendarGenerator("islamic", "_IslamicLunar")
    cal_gen.generate(dates)


if __name__ == "__main__":
    generate_data()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.ecuador import Ecuador
from tests.common import CommonCountryTests


class TestEcuador(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Ecuador)

    def test_new_years_day(self):
        name = "Año Nuevo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival(self):
        name = "Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
            "2024-02-12",
            "2024-02-13",
            "2025-03-03",
            "2025-03-04",
        )
        self.assertHolidayNameCount(name, 2, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Día del Trabajo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2018-04-30",
            "2019-05-03",
            "2021-04-30",
            "2022-05-02",
            "2024-05-03",
            "2025-05-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_battle_of_pichincha(self):
        name = "Batalla de Pichincha"
        self.assertHolidayName(name, (f"{year}-05-24" for year in self.full_range))
        obs_dts = (
            "2017-05-26",
            "2018-05-25",
            "2020-05-25",
            "2022-05-23",
            "2023-05-26",
            "2025-05-23",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_of_quito(self):
        name = "Primer Grito de Independencia"
        self.assertHolidayName(name, (f"{year}-08-10" for year in self.full_range))
        obs_dts = (
            "2017-08-11",
            "2019-08-09",
            "2021-08-09",
            "2022-08-12",
            "2023-08-11",
            "2024-08-09",
            "2025-08-11",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_of_guayaquil(self):
        name = "Independencia de Guayaquil"
        self.assertHolidayName(name, (f"{year}-10-09" for year in self.full_range))
        obs_dts = (
            "2018-10-08",
            "2019-10-11",
            "2021-10-08",
            "2022-10-10",
            "2024-10-11",
            "2025-10-10",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_all_souls_day(self):
        name = "Día de Difuntos"
        self.assertHolidayName(name, (f"{year}-11-02" for year in self.full_range))
        obs_dts = (
            "2019-11-01",
            "2021-11-01",
            "2022-11-04",
            "2024-11-01",
            "2025-11-04",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_of_cuenca(self):
        name = "Independencia de Cuenca"
        self.assertHolidayName(name, (f"{year}-11-03" for year in self.full_range))
        obs_dts = (
            "2019-11-04",
            "2024-11-04",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Navidad"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2021-12-24",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-28", "Carnaval"),
            ("2022-03-01", "Carnaval"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-02", "Día del Trabajo (observado)"),
            ("2022-05-23", "Batalla de Pichincha (observado)"),
            ("2022-05-24", "Batalla de Pichincha"),
            ("2022-08-10", "Primer Grito de Independencia"),
            ("2022-08-12", "Primer Grito de Independencia (observado)"),
            ("2022-10-09", "Independencia de Guayaquil"),
            ("2022-10-10", "Independencia de Guayaquil (observado)"),
            ("2022-11-02", "Día de Difuntos"),
            ("2022-11-03", "Independencia de Cuenca"),
            ("2022-11-04", "Día de Difuntos (observado)"),
            ("2022-12-25", "Navidad"),
            ("2022-12-26", "Navidad (observado)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Carnival"),
            ("2022-03-01", "Carnival"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day (observed)"),
            ("2022-05-23", "The Battle of Pichincha (observed)"),
            ("2022-05-24", "The Battle of Pichincha"),
            ("2022-08-10", "Declaration of Independence of Quito"),
            ("2022-08-12", "Declaration of Independence of Quito (observed)"),
            ("2022-10-09", "Independence of Guayaquil"),
            ("2022-10-10", "Independence of Guayaquil (observed)"),
            ("2022-11-02", "All Souls' Day"),
            ("2022-11-03", "Independence of Cuenca"),
            ("2022-11-04", "All Souls' Day (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-28", "Карнавал"),
            ("2022-03-01", "Карнавал"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-05-01", "День праці"),
            ("2022-05-02", "День праці (вихідний)"),
            ("2022-05-23", "День битви біля Пічинча (вихідний)"),
            ("2022-05-24", "День битви біля Пічинча"),
            ("2022-08-10", "День незалежності Кіто"),
            ("2022-08-12", "День незалежності Кіто (вихідний)"),
            ("2022-10-09", "День незалежності Гуаякіля"),
            ("2022-10-10", "День незалежності Гуаякіля (вихідний)"),
            ("2022-11-02", "День усіх померлих"),
            ("2022-11-03", "День незалежності Куенки"),
            ("2022-11-04", "День усіх померлих (вихідний)"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Різдво Христове (вихідний)"),
        )
